# -*- coding: utf-8 -*-
# Copyright 2011 Thamini S.à.R.L    This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import time
from datetime import date
from osv import osv
from osv import fields
from tools.misc import DEFAULT_SERVER_DATE_FORMAT
import decimal_precision as dp

class horse_function__advance(osv.osv):
    _name = 'horse.function'
horse_function__advance()

class horse_discipline(osv.osv):
    _name = 'horse.discipline'
    _columns = {
        'name': fields.char('Name', size=32),
        'description': fields.text('Description'),
    }
horse_discipline()

class horse_breed(osv.osv):
    _name = 'horse.breed'
    _columns = {
        'name': fields.char('Name', size=64, required=True, translate=True),
    }
horse_breed()

class horse_coat(osv.osv):
    _name = 'horse.coat'
    _columns = {
        'name': fields.char('Name', size=64, required=True, translate=True),
    }
horse_coat()

class horse_horse(osv.osv):
    _name = 'horse.horse'

    def _calc_age(self, horse_birthday):
        if not horse_birthday:
            return 0
        today = date.today()
        birthday = date(*time.strptime(horse_birthday, DEFAULT_SERVER_DATE_FORMAT)[:3])
        year_diff = today.year - birthday.year
        try:
            birthday.replace(year=today.year)
        except ValueError:
            if birthday.month == 2 and birthday.day == 29:
                # in case the birthday is one the 29 Feb.
                birthday.replace(year=today.year, day=birthday.day-1)
        if birthday < today:
            # this year birthdate before today
            return year_diff
        else:
            return max(0, year_diff-1)

    def _calc_horse_age(self, cr, uid, ids, fname, args, context=None):
        today = date.today()
        result = {}
        for horse in self.browse(cr, uid, ids, context=context):
            result[horse.id] = self._calc_age(horse.birth_date)
        return result

    _columns = {
        'name': fields.char('Name', size=64, required=True),
        'chipid': fields.char('Chip ID', size=128, required=True),
#        'breed': fields.char('Breed', size=64),
        'breed_id': fields.many2one('horse.breed', 'Breed'),
#        'coat': fields.char('Coat', size=64),
        'coat_id': fields.many2one('horse.coat', 'Coat'),
        'birth_date': fields.date('Birth Date'),
        'age': fields.function(_calc_horse_age, type='integer', string='Age', method=True, readonly=True),
        'stud_book_id': fields.char('Stud-Book ID', size=32),
        'gender': fields.selection([('male','Male'),('female','Female'),('gelding','Gelding')], 'Gender'),
        'note': fields.text('Note'),
        'function_ids': fields.one2many('horse.function', 'horse_id', 'Functions'),
    }

    _sql_constraints = [
        ('uniq_chipid', 'UNIQUE(chipid)', 'Chip ID must be unique'),
    ]

    def name_get(self, cr, user, ids, context=None):
        if not context:
            context = {}
        if not ids:
            return []
        if isinstance(ids, (int, long)):
            ids = [ ids ]
        return [ (r['id'], u'%s (%s)' % (r[self._rec_name], r['chipid'])) \
                    for r in self.read(cr, user, ids, [self._rec_name, 'chipid'], context, load='_classic_write') ]

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        if args is None:
            args = []
        if name:
            args = ['|',('chipid',operator,name)] + args
        return super(horse_horse, self).name_search(cr, user, name, args, operator, context, limit)


horse_horse()

class horse_function_kind(osv.osv):
    _name = 'horse.function.kind'
    _columns = {
        'code': fields.char('Code', size=32),
        'name': fields.char('Name', size=64, translate=True),
    }
horse_function_kind()

class horse_function(osv.osv):
    _name = 'horse.function'
    _rec_name = 'horse_id'

    _states_selection = [
        ('current', 'Current'),
        ('past', 'Past'),
    ]

    def _get_kinds_selection(self, cr, uid, context=None):
        kind_proxy = self.pool.get('horse.function.kind')
        kind_ids = kind_proxy.search(cr, 1, [], context=context)
        return [ (x['code'], x['name']) for x in kind_proxy.read(cr, uid, kind_ids, context=context) ]

    _columns = {
        'horse_id': fields.many2one('horse.horse', 'Horse', required=True, ondelete='cascade'),
        'partner_id': fields.many2one('res.partner', 'Partner', required=True),
        'address_id': fields.many2one('res.partner.address', 'Address', required=True, domain="[('partner_id','=',partner_id)]"),
        #'kind': fields.selection(_kinds_selection, 'Kind', required=True),
        'kind': fields.selection(_get_kinds_selection, 'Kind', required=True),
        'rider_license': fields.char('Rider License', size=64),
        'purchase_date': fields.date('Purchase Date'),
        'purchase_price': fields.float('Purchase Price', digits_compute=dp.get_precision('Account')),
        'discipline_id': fields.many2one('horse.discipline', 'Discipline'),
        'start_date': fields.date('Start Date'),
        'end_date': fields.date('End Date'),
        'state': fields.selection(_states_selection, 'State', required=True),
    }

    _defaults = {
        'state': 'current',
    }

    def onchange_partner_id(self, cr, uid, ids, partner_id, context=None):
        ocv = {'value': {'address_id': False }}
        if partner_id:
            partner_proxy = self.pool.get('res.partner')
            partner = partner_proxy.browse(cr, uid, partner_id, context=context)
            if partner.address:
                ocv['value']['address_id'] = partner.address[0].id
        return ocv

horse_function()

class res_partner_horse(osv.osv):
    _inherit = 'res.partner'
    _columns = {
        'horse_function_ids': fields.one2many('horse.function', 'partner_id', 'Horse Functions'),
    }
res_partner_horse()

class res_partner_address_horse(osv.osv):
    _inherit = 'res.partner.address'
    _columns = {
        'horse_function_ids': fields.one2many('horse.function', 'address_id', 'Horse Functions'),
    }
res_partner_address_horse()
